#include <iostream>

#include "eval.h"
#include "defs.h"
#include "pawntab.h"
#include "squares.h"
#include "bits.h"

//from material analyses using Stats, using regression for influence of majors and pawns

//BPair
const int Bpairmajors[7] = {23,46,69,92,115,138,161};
const int Bpairpawns[17] = {-27,-53,-80,-106,-133,-160,-186,-213,-240,-266,-293,-319,-346,-373,-399,-426};
const int BpairM = 183*256/100;

//Bvs3P - Not many games from database, and the black and white stats were polar opposites
//num majors
const int Bv3Pmaj[7] = {166,171,175,179,184,188,192};
const int Bv3Pmaterial = vB-3*vP; //standard material advantage for N vs 3P


//Nvs3P - No games to see how reduced pawns influences the situation
//num majors
const int Nv3Pmaj[7] = {-51,-16,19,54,89,124,159};
const int Nv3Pmaterial = vN-3*vP; //standard material advantage for N vs 3P

//Exchange
const int exchBR = vB-vR+361; //this means less advantage than the normal material advantage
const int exchNR = vN-vR+445;


//NvsB
const int NvsBpawns[17] = {0,15,30,45,59,74,89,104,119,134,148,163,178,193,208,223,238};
const int NvsBmaj[7] = {0,3,7,10,14,17,21};
const int NvsBM = -83*256/100;

//2NvsB
const int NNvsB = 2*vP-vN; //table said total adv = 2P


//endgame with no pawns, check for a material draw
void cEval::balance(cBoard &brd, cMaterial &mat)
{
    int wp = mat.getpcenum(pwP);
    int wq = mat.getpcenum(pwQ);
    int wr = mat.getpcenum(pwR);
    int wb = mat.getpcenum(pwB);
    int wn = mat.getpcenum(pwN);
    int bp = mat.getpcenum(pbP);
    int bq = mat.getpcenum(pbQ);
    int br = mat.getpcenum(pbR);
    int bb = mat.getpcenum(pbB);
    int bn = mat.getpcenum(pbN);

    if((wp+bp)==0)
    {
        if(is_draw(brd,mat))
        {
            drawflag = true;
        }
    }

    uint bmaj = bq+br;
    uint wmaj = wq+wr;
    uint wmin = wb+wn;
    uint bmin = bn+bb;

    if(bmaj>3) bmaj=3;
    if(wmaj>3) wmaj=3;

    int tempb=0,tempw=0;

    ASS(bmaj+wmaj>=0 && bmaj+wmaj<=6);
    ASS(wp<9 && bp<9);
    ASS(wp>=0 && bp>=0);

    //Bishop Pair**************************
    if(wb >= 2 && bb < 2)
    {
        tempw += Bpairmajors[bmaj+wmaj];
        tempw += Bpairpawns[wp+bp];
        tempw += BpairM;
/*
        brd.printboard();
        cout<<"\n WB Pair ";
        cout<<"\n bonus "<<cpscore((Bpairmajors[bmaj+wmaj]+Bpairpawns[wp+bp]));
        system("PAUSE");
*/
    }
    else if(bb >= 2 && wb < 2)
    {
        tempb += Bpairmajors[bmaj+wmaj];
        tempb += Bpairpawns[wp+bp];
        tempb += BpairM;
    }

    //Bv3P********************************
    //white
    if(wmaj==bmaj && bp-wp==3 && wmin-bmin==1 && wb>bb)
    {
        ASS(bp+wp<=10);
        tempw += Bv3Pmaj[bmaj+wmaj];
        tempw -= Bv3Pmaterial;
/*
        brd.printboard();
        cout<<"\n whiteBv3P ";
        cout<<"\n bonus "<<cpscore(Bv3Ppns[wp+bp]+Bv3Pmaj[bmaj+wmaj]-Bv3Pmaterial);
        system("PAUSE");
*/
    }

    //black
    if(wmaj==bmaj && wp-bp==3 && bmin-wmin==1 && bb>wb)
    {
        ASS(bp+wp<=10);
        tempb += Bv3Pmaj[bmaj+wmaj];
        tempb -= Bv3Pmaterial;
    }

    //Nv3P********************************
    //white
    if(wmaj==bmaj && bp-wp==3 && wmin-bmin==1 && wn>bn)
    {
        ASS(bp+wp<=10);
        tempw += Nv3Pmaj[bmaj+wmaj]-Nv3Pmaterial;
/*
        brd.printboard();
        cout<<"\n white Nv3P ";
        cout<<"\n bonus "<<cpscore(Nv3Pmaj[bmaj+wmaj]-Nv3Pmaterial);
        system("PAUSE");
*/
    }

    //black
    if(wmaj==bmaj && wp-bp==3 && bmin-wmin==1 && bn>wn)
    {
        ASS(bp+wp<=10);
        tempb += Nv3Pmaj[bmaj+wmaj]-Nv3Pmaterial;
    }

    //Exchange*****************************
    //RforN
    if(wr-br==1 && bmin-wmin==1 && bn>wn) tempw += exchNR;
    if(br-wr==1 && wmin-bmin==1 && wn>bn) tempb += exchNR;
    //RforB
    if(wr-br==1 && bmin-wmin==1 && bb>wb) tempw += exchBR;
    if(br-wr==1 && wmin-bmin==1 && wb>bb) tempb += exchBR;


    //NvsB*********************************
    if(wmaj==bmaj && wmin==bmin)
    {
        if(wn>bb)
        {
         tempw += NvsBpawns[wp+bp];
         tempw += NvsBmaj[wmaj+bmaj];
         tempw += NvsBM;
        }
        else if(bn>wn)
        {
         tempb += NvsBpawns[wp+bp];
         tempb += NvsBmaj[wmaj+bmaj];
         tempb += NvsBM;
        }
    }

    //NN vs B************************************
    if(wmaj==bmaj && wmin-bmin==1 && wn-bn==2) tempw += NNvsB;
    if(wmaj==bmaj && bmin-wmin==1 && bn-wn==2) tempw += NNvsB;



    bal[cW][OPE]+= tempw;
    bal[cW][END]+= tempw;

    bal[cB][OPE]+= tempb;
    bal[cB][END]+= tempb;
}

bool cEval::is_draw(cBoard &brd, cMaterial &mat)
{
    int wn = mat.getpcenum(pwN);
    int wr = mat.getpcenum(pwR);
    int wb = mat.getpcenum(pwB);
    int wq = mat.getpcenum(pwQ);
    int bn = mat.getpcenum(pbN);
    int br = mat.getpcenum(pbR);
    int bb = mat.getpcenum(pbB);
    int bq = mat.getpcenum(pbQ);

    if (!wr && !br && !wq && !bq)
	{
	  if (!bb && !wb)
	  {
	      if (wn < 3 && bn < 3) {  return true; }
	   }
	  else if (!wn && !bn)
	  {
	     if (abs(wb - bb) < 2) { return true; }
	  }
	  else if ((wn < 3 && !wb) || (wb == 1 && !wn))
	  {
	    if ((bn < 3 && !bb) || (bb == 1 && !bn))  { return true; }
	  }
	}
   else if (!wq && !bq)
   {
	if (wr == 1 && br == 1)
	{
	  if ((wn + wb) < 2 && (bn + bb) < 2)	{ return true; }
	}
	else if (wr == 1 && !br)
	{
	 if ((wn + wb == 0) && (((bn + bb) == 1) || ((bn + bb) == 2))) { return true; }
    }
	else if (br == 1 && !wr)
	{
	 if ((bn + bb == 0) && (((wn + wb) == 1) || ((wn + wb) == 2))) { return true; }
     }

    }
  return false;
}


